@@ -1,10 +1,7 @@
 # Author: Ingo Feinerer
 
-setClass("Dictionary",
-         contains = "character")
-
 Dictionary <- function(x) UseMethod("Dictionary", x)
 Dictionary.character <- function(x)
-    new("Dictionary", .Data = x)
+    structure(x, class = c("Dictionary", "character"))
 Dictionary.TermDocumentMatrix <- Dictionary.DocumentTermMatrix <- function(x)
-    new("Dictionary", .Data = Terms(x))
+    structure(Terms(x), class = c("Dictionary", "character"))
