@@ -36,64 +36,90 @@ weightTfIdf <-
         if (isDTM) t(m) else m
     }, "term frequency - inverse document frequency", "tf-idf")
 
-# Not yet finished
 weightSMART <-
     WeightFunction(function(m, spec = "nnn", control = list()) {
-        if (nchar(spec) != 3)
+        if (nchar(spec) != 3L)
             stop("invalid spec")
 
-        term_frequency <- match.arg(substr(spec, 1, 1), c("n", "l", "a", "b", "L"))
-        document_frequency <- match.arg(substr(spec, 2, 2), c("n", "t", "p"))
-        normalization <- match.arg(substr(spec, 3, 3), c("n", "b"))
+        term_frequency <-
+            match.arg(substr(spec, 1L, 1L),
+                      c("n", "l", "a", "b", "L"))
+        document_frequency <-
+            match.arg(substr(spec, 2L, 2L),
+                      c("n", "t", "p"))
+        normalization <-
+            match.arg(substr(spec, 3L, 3L),
+                      c("n", "c", "u", "b"))
 
         isDTM <- inherits(m, "DocumentTermMatrix")
         if (isDTM) m <- t(m)
 
-        # Term frequency
-        m$v <- switch(term_frequency,
-                      # natural
-                      n = m$v,
-                      # logarithm
-                      l = 1 + log2(m$v),
-                      # augmented
-                      a = 0.5 + (0.5 * m$v) / tapply(m$v, m$j, max),
-                      # boolean
-                      b = rep(1, length(m$v)),
-                      # log ave
-                      # TODO: What does avg_{t \in d} in Manning et al. mean?
-                      L = (1 + log2(m$v)) / (1 + log2(tapply(m$v, m$j, mean))))
+        if(normalization == "b") {
+            ## Need to compute the character lenghts of the documents
+            ## before starting the weighting.
+            charlengths <-
+                tapply(nchar(Terms(m))[m$i] * m$v, m$j, sum)
+        }
 
-        # Document frequency
+        ## Term frequency
+        m$v <- switch(term_frequency,
+                      ## natural
+                      n = m$v,
+                      ## logarithm
+                      l = 1 + log2(m$v),
+                      ## augmented
+                      a = {
+                          s <- tapply(m$v, m$j, max)
+                          0.5 + (0.5 * m$v) / s[as.character(m$j)]
+                      },
+                      ## boolean
+                      b = as.numeric(m$v > 0),
+                      ## log ave
+                      L = {
+                          s <- tapply(m$v, m$j, mean)
+                          ((1 + log2(m$v)) /
+                           (1 + log2(s[as.character(m$j)])))
+                      })
+
+        ## Document frequency
         rs <- row_sums(m > 0)
         if (any(rs == 0))
             warning("term does not occur in the corpus")
         df <- switch(document_frequency,
-                     # natural
+                     ## natural
                      n = 1,
-                     # idf
+                     ## idf
                      t = log2(nDocs(m) / rs),
-                     # prob idf
+                     ## prob idf
                      p = max(0, log2((nDocs(m) - rs) / rs)))
 
-        # Normalization
+        ## Normalization
         norm <- switch(normalization,
-                       # none
+                       ## none
                        n = rep(1, nDocs(m)),
-                       # cosine
-                       # Not implemented yet.
-                       # pivoted unique
-                       # Not implemented yet.
-                       # byte size
+                       ## cosine
+                       c = sqrt(col_sums(m ^ 2)),
+                       ## pivoted unique
+                       u = {
+                           if(is.null(pivot <- control$pivot))
+                               stop("invalid control argument pivot")
+                           if(is.null(slope <- control$slope))
+                               stop("invalid control argument slope")
+                           (slope * sqrt(col_sums(m ^ 2)) +
+                            (1 - slope) * pivot)
+                       },
+                       ## byte size
                        b = {
-                           if (is.null(control$charlength))
-                               stop("invalid control argument charlength")
-                           if (is.null(control$charlength))
+                           if(is.null(alpha <- control$alpha))
                                stop("invalid control argument alpha")
-                           control$charlength^control$alpha
+                           norm <- double(nDocs(m))
+                           norm[match(names(charlengths),
+                                      seq_along(norm))] <-
+                                          charlengths ^ alpha
+                           norm
                        })
 
         m <- m * df
-        names(norm) <- seq_len(nDocs(m))
         m$v <- m$v / norm[m$j]
         attr(m, "Weighting") <- c(paste("SMART", spec), "SMART")
 
